@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014 Brian L. Browning
+ * Copyright (C) 2014-2016 Brian L. Browning
  *
  * This file is part of Beagle
  *
@@ -38,10 +38,8 @@ import java.util.Set;
  */
 public class BasicMarker implements Marker {
 
-    private static final int MIN_NUMBER_FIELDS = 8;
-
     private static final String[] EMPTY_ID_ARRAY = new String[0];
-    private static final Map<String, String[]> allelesMap
+    private static final Map<String, String[]> ALLELES_MAP
             = new HashMap<>(24);
 
     private final int chromIndex;
@@ -100,7 +98,7 @@ public class BasicMarker implements Marker {
         if (chrom<0 || chrom>=ChromIds.instance().size()) {
             throw new IndexOutOfBoundsException(String.valueOf(chrom));
         }
-        checkIds(chrom, pos, ids);
+        ids = removeMissingIds(chrom, pos, ids);
         checkAlleles(chrom, pos, alleles);
         checkEnd(chrom, pos, end);
         this.chromIndex = chrom;
@@ -111,21 +109,27 @@ public class BasicMarker implements Marker {
         this.end = end;
     }
 
-    private static void checkIds(int chrom, int pos, String[] ids) {
-        for (String id : ids) {
-            for (int j=0, n=id.length(); j<n; ++j) {
-                if (Character.isWhitespace(id.charAt(j))) {
+    private static String[] removeMissingIds(int chrom, int pos, String[] ids) {
+        int index = 0;
+        for (int j=0; j<ids.length; ++j) {
+            String id = ids[j];
+            for (int k=0, n=id.length(); k<n; ++k) {
+                if (Character.isWhitespace(id.charAt(k))) {
                     String s = "ERROR: ID field contains white-space at "
                             + coordinate(chrom, pos) + " [" + id + "]";
                     Utilities.exit(s);
                 }
             }
+            if (id.length()>0 && id.equals(Const.MISSING_DATA_STRING)==false) {
+                ids[index++] = id;
+            }
         }
+        return (index < ids.length) ? Arrays.copyOf(ids, index) : ids;
     }
 
     private static void checkAlleles(int chrom, int pos, String[] alleles) {
-        if (alleles.length<2) {
-            String s = "ERROR: missing REF or ALT allele at "
+        if (alleles.length<1) {
+            String s = "ERROR: missing REF allele at "
                     + coordinate(chrom, pos);
             throw new IllegalArgumentException(s);
         }
@@ -192,12 +196,12 @@ public class BasicMarker implements Marker {
             for (int j=1; j<alleles.length; ++j) {
                 key += alleles[j];
             }
-            String[] storedAlleles = allelesMap.get(key);
+            String[] storedAlleles = ALLELES_MAP.get(key);
             if (storedAlleles!=null) {
                 alleles = storedAlleles;
             }
             else {
-                allelesMap.put(key, alleles.clone());
+                ALLELES_MAP.put(key, alleles.clone());
             }
         }
         return alleles;
@@ -222,21 +226,21 @@ public class BasicMarker implements Marker {
 
     /**
      * Constructs a new {@code BasicMarker} instance from the specified
-     * string VCF record.
-     * @param vcfRecord a VCF record
-     * @throws IllegalArgumentException if the specified VCF
-     * record has fewer than 8 tab-delimited fields, or if a format
-     * error is detected in the specified VCF record
+     * string VCF record prefix.
+     * @param vcfRecord a VCF record prefix
+     * @throws IllegalArgumentException if the specified VCF record
+     * prefix has fewer than 8 tab-delimited fields, or if a format
+     * error is detected in the first 8 fields of the specified VCF record
      * @throws NullPointerException if {@code vcfRecord == null}
      */
     @SuppressWarnings("RedundantStringConstructorCall")
     public BasicMarker(String vcfRecord) {
+        int minFieldCnt = 8;
         String[] fields = StringUtil.getFields(vcfRecord, Const.tab,
-                MIN_NUMBER_FIELDS+1);
-        if (fields.length < MIN_NUMBER_FIELDS) {
+                minFieldCnt+1);
+        if (fields.length < minFieldCnt) {
             String s = "VCF record does not contain at least "
-                    + MIN_NUMBER_FIELDS + " tab-delimited fields: "
-                    + vcfRecord;
+                    + minFieldCnt + " tab-delimited fields: " + vcfRecord;
             Utilities.exit(s);
         }
         // Store minimal required data, not entire VCF record
@@ -288,7 +292,7 @@ public class BasicMarker implements Marker {
             return EMPTY_ID_ARRAY;
         }
         String[] ids = StringUtil.getFields(id, Const.semicolon);
-        checkIds(chrom, pos, ids);
+        removeMissingIds(chrom, pos, ids);
         return ids;
     }
 
@@ -364,7 +368,7 @@ public class BasicMarker implements Marker {
         this.alleles = new String[m.nAlleles()];
         for (int j=0; j<this.alleles.length; ++j) {
             if (m.allele(j).charAt(0)!='<') {    // not a symbolic allele
-                this.alleles[j] = flipAllele(m.allele(j));
+                this.alleles[j] = flipStrand(m.allele(j));
             }
             else {
                 this.alleles[j] = m.allele(j);
@@ -374,7 +378,17 @@ public class BasicMarker implements Marker {
         this.end = m.end();
     }
 
-    private static String flipAllele(String allele) {
+    /**
+     * Returns the string allele obtained by changing the specified allele
+     * to the opposite chromosome strand
+     * @param allele a string allele
+     * @return the string allele obtained by changing the specified allele
+     * to the opposite chromosome strand
+     * @throws IllegalArgumentException if any character in the specified
+     * string is not 'A', 'C', 'G', 'T', 'N', or '*'.
+     * @throws NullPointerException if {@code allele == null}
+     */
+    public static String flipStrand(String allele) {
         char[] ca = new char[allele.length()];
         for (int j=0; j<ca.length; ++j) {
             ca[j] = flipBase(allele.charAt(j));
@@ -390,7 +404,7 @@ public class BasicMarker implements Marker {
             case 'T' : return 'A';
             case 'N' : return 'N';
             case '*' : return '*';
-            default: assert false; return 0;
+            default: throw new IllegalArgumentException(String.valueOf(c));
         }
     }
 
